/* fileio.c */


/* Library of file I/O functions usable on CRAY, SGI, Stellar, etc. */


#include <stdio.h>




/*** read_int4 ********************************************************
   Read a 4-byte integer from a file.
   Input:  f - the file to read from
           i - pointer to integer to put result into.
   Returned:  0 = error, 1 = success
**********************************************************************/
int read_int4( f, i )
FILE *f;
int *i;
{
   if ( fread( i, 4, 1, f ) ) {
#ifdef cray
      *i = *i >> 32;
#endif
      return 1;
   }
   return 0;
}



/*** read_float4 ******************************************************
   Read a 4-byte IEEE float from a file.
   Input:  f - the file to read from.
           x - pointer to float to put result into.
   Returned:  0 = error, 1 = success
**********************************************************************/
int read_float4( f, x )
FILE *f;
float *x;
{
#ifdef cray
   long buffer = 0;

   if ( fread( &buffer, 4, 1, f ) ) {
      /* convert IEEE float (buffer) to Cray float (x) */
      IF_TO_C( x, &buffer );
      return 1;
    }
    return 0;
#else
   if ( fread( x, 4, 1, f ) ) {
      return 1;
    }
    return 0;
#endif
}



/*** write_int4 *******************************************************
   Write a 4-byte integer to a file.
   Input:  f - the file
           i - the integer
   Return:  0 = error, 1 = success
**********************************************************************/
int write_int4( f, i )
FILE *f;
int i;
{
#ifdef cray
   i = i << 32;
   return fwrite( &i, 4, 1, f ) > 0;
#else
   return fwrite( &i, 4, 1, f ) > 0;
#endif
}




/*** write_float4 *****************************************************
   Write a 4-byte IEEE float to a file.
   Input:  f - the file
           x - the float
   Return:  0 = error, 1 = success
**********************************************************************/
int write_float4( f, x )
FILE *f;
float x;
{
#ifdef cray
   char buffer[8];
   C_TO_IF( buffer, &x );
   return fwrite( buffer, 4, 1, f ) > 0;
#else
   float y;
   y = (float) x;
   return fwrite( &y, 4, 1, f ) > 0;
#endif
}



/*** write_int4_array *************************************************
   Write an array of 'n' 4-byte integers to a file.
   Input:  f - the file
           i - the array of ints
	   n - the number of ints in array
   Return:  0 = error, 1 = success
**********************************************************************/
int write_int4_array( f, i, n )
FILE *f;
int i[];
{
#ifdef cray
   int j, result;
   char *buf, *b, *ptr;

   b = buf = (char *) malloc( n*4 );
   if (!b)
      return 0;
   ptr = (char *) i;
   for (j=0;j<n;j++) {
      ptr += 4;      /* skip upper 4 bytes */
      *b++ = *ptr++;
      *b++ = *ptr++;
      *b++ = *ptr++;
      *b++ = *ptr++;
   }
   result = fwrite( buf, 4, n, f ) > 0;
   free( buf );
   return result;
#else
   return fwrite( i, 4, n, f) > 0;
#endif
}



/*** write_float4_array ***********************************************
   Write an array of 'n' 4-byte IEEE floating point numbers to a file.
   Input:  f - the file
           x - the array of floats
           n - number of floats in array
   Return:  0 = error, 1 = success.
**********************************************************************/
int write_float4_array( f, x, n )
FILE *f;
float x[];
int n;
{
#ifdef cray
   int j, result;
   long *b, *buf;
   long tmp;

   /* convert cray floats to IEEE and put into buffer */
   b = buf = (long *) malloc( n*4 );
   if (!b)
      return 0;
   for (j=0;j<n;j++) {
      C_TO_IF( &tmp, &x[j] );
      if (j&1) {
	 *b = *b | (tmp >> 32);
	 b++;
      }
      else {
	 *b = tmp;
      }
   }
   result = fwrite( buf, 4, n, f) > 0;
   free( buf );
   return result;
#else
   return fwrite( x, 4, n, f) > 0;
#endif
}



/*** read_int4_array **************************************************
   Read an array of 'n' 4-byte integers from a file.
   Input:  f - the file
           i - the array of ints
	   n - the number of ints to read
   Return:  0 = error, 1 = success
**********************************************************************/
int read_int4_array( f, i, n )
FILE *f;
int i[];
{
#ifdef cray
   int j;
   char *buf, *b, *ptr;

   b = buf = (char *) malloc( n*8 );
   if (!buf)
      return 0;
   if (fread( buf, 4, n, f )==n) {
      ptr = (char *) i;
      for (j=0;j<n;j++) {
	 b += 4;
	 *ptr++ = *b++;
	 *ptr++ = *b++;
	 *ptr++ = *b++;
	 *ptr++ = *b++;
      }
      free( buf );
      return 1;
   }
   return 0;
#else
   return fread( i, 4, n, f)==n;
#endif
}






/*** write_bytes ******************************************************
   Write an block of 'n' bytes to a file.
   Input:  f - the file
           b - pointer to buffer
	   n - number of bytes to write
   Return:  0 = error, 1 = success
**********************************************************************/
int write_bytes( f, b, n )
FILE *f;
char *b;
int n;
{
   return fwrite( b, 1, n, f ) > 0;
}



/*** read_bytes *******************************************************
   Read a block of 'n' bytes from a file.
   Input:  f - the file
           b - pointer to buffer
	   n - number of bytes to read
   Return:  0 = error, 1 = success
**********************************************************************/
int read_bytes( f, b, n )
FILE *f;
char *b;
int n;
{
   return fread( b, 1, n, f ) > 0;
}
